common = {

    configuration: [
        {
            'name': 'indent',
            'displayName': 'Indent',
            'type': 'enum',
            'default': 'tab',
            'values': {
                'none': 'none',
                'space2': '2 spaces',
                'space4': '4 spaces',
                'tab': 'tab'
            }
        },
        {
            'name': 'format',
            'displayName': 'Format',
            'type': 'enum',
            'default': 'json',
            'values': {
                'plaintext-json': 'JSON with plaintext keys, flat view',
                'json': 'JSON with namespaces in keys, tree view',
                'namespaced-json': 'JSON with nested namespaces, tree view'
            }
        }
    ],


    formats: {
        'plaintext-json': {
            'flags': {
                'nested-keys': false,
                'split-join-keys': false
            }
        },
        'json': {
            'flags': {
                'nested-keys': true,
                'split-join-keys': true
            }
        },
        'namespaced-json': {
            'flags': {
                'nested-keys': true,
                'split-join-keys': false
            }
        }
    },


    //
    // Imports file contents
    // filecontent: content of the file as string
    // language: the language for this file (might be empty if one file contains all languages)
    //
    importData: function(filecontent, fileformat)
    {
        if(babeledit.checkAndReportJsonError(filecontent, 0))
        {
            return;
        }

        var data = JSON.parse(filecontent);

        if (!fileformat)
        {
            fileformat = common._guessFormat(data);
            if (fileformat)
            {
                babeledit.setFileFormat(fileformat);
            }
        }
        if (fileformat)
        {
            this.formatFlags = common.formats[fileformat].flags;
        }
        else
        {
            // format flags doesn't matter, as keys do neither contain delimiters nor nested json objects
            this.formatFlags = common.formats['json'].flags;
        }

        common._recursiveCollectTranslations([], data, null);
    },


    _guessFormat: function(rootNode)
    {
        var delimiterFound = false;
        for (var key in rootNode)
        {
            if (typeof rootNode[key] === "object")
            {
                return "namespaced-json";
            }
            if (typeof rootNode[key] === "string" &&
                (key.includes(". ") || key.includes(" .") || key.includes("..") ||
                 key.endsWith(".") || key.endsWith("!") || key.endsWith("?")))
            {
                return "plaintext-json";
            }
            delimiterFound = delimiterFound || key.includes(info.delimiter);
        }
        return delimiterFound ? "json" : null;
    },


    _recursiveCollectTranslations: function(parentPath, subtree, language)
    {
        for(var key in subtree)
        {
            var fullPath;
            if (this.formatFlags['split-join-keys'])
            {
                // split path using delimiter if needed
                fullPath = parentPath.concat(key.split(info.delimiter));
            }
            else
            {
                fullPath = parentPath.concat([key]);
            }

            var object = subtree[key];
            if(typeof object === "string")
            {
                // if nested-keys is false (plaintext-json) we automatically flatten all nested translation ids,
                if (!this.formatFlags['nested-keys'] && fullPath.length > 1)
                {
                    fullPath = [ fullPath.join(info.delimiter) ];
                }

                // leaf found: add translation
                babeledit.addTranslation(fullPath, language, object);
            }
            else
            {
                // still in tree - recurse
                common._recursiveCollectTranslations(fullPath, object, language);
            }
        }
    },

    prepareExport: function(filename, language, configuration)
    {
        this.config = configuration;
        this.languageToExport = language;
        this.documentRoot = {};
        this.formatFlags = common.formats[configuration.format].flags;
    },

    //
    // save translation for current file
    //
    saveTranslation: function(fullPath, language, translation)
    {
        if(language !== this.languageToExport)
        {
            return;
        }

        // if nested-keys is false (plaintext-json) the join() shouldn't be necessary,
        // as fullPath may only contain one element
        if(this.formatFlags['split-join-keys'] ||
           (!this.formatFlags['nested-keys'] && fullPath.length > 1))
        {
            this.documentRoot[fullPath.join(info.delimiter)] = translation;
        }
        else
        {
            // namespaced json
            var key = fullPath.pop();
            var treeNode = common._makePathRecursive(fullPath, this.documentRoot);
            treeNode[key] = translation;
        }
    },

    //
    // return formatted file data
    //
    getSaveFile: function()
    {
        var indent;

        this.documentRoot = common._makeArraysRecursive(this.documentRoot);

        switch(this.config.indent)
        {
            case 'space2':
                indent = "  ";
                break;
            case 'space4':
                indent = "    ";
                break;
            case 'tab':
                indent = "\t";
                break;
        }

        return JSON.stringify(this.documentRoot, null, indent) + "\n";
    },


    _makeArraysRecursive: function(root)
    {
        var res = {};
        for(var key in root)
        {
            if (typeof root[key] === "object")
            {
                if(common._allKeysAreNumbersAndAllItemsAreStrings(root[key]))
                {
                    res[key] = common._objectToArray(root[key]);
                }
                else
                {
                    res[key] = common._makeArraysRecursive(root[key]);
                }
            }
            else
            {
                res[key] = root[key];
            }
        }
        return res;
    },


    _allKeysAreNumbersAndAllItemsAreStrings: function(root)
    {
        var count = Object.keys(root).length;
        for(var i=0; i<count; i++)
        {
            if(typeof root[''+i] !== "string")
            {
                return false;
            }
        }
        return true;
    },


    _objectToArray: function(root)
    {
        var array = [];
        for(var key in root)
        {
            array[key] = root[key];
        }
        return array;
    },


    _makePathRecursive: function(path, tree)
    {
        if(!path || path.length === 0)
        {
             return tree;
        }

        var key = path.shift();
        if(!tree.hasOwnProperty(key))
        {
            tree[key] = {};
        }
        return common._makePathRecursive(path, tree[key]);
    }

}
